﻿'Coded by ShadowTzu
'Free to use

'my 3D engine, Tzu3D: http://shadowtzu.free.fr
'Youtube: https://www.youtube.com/user/shadowtzu
'Facebook: https://www.facebook.com/Tzu3d
'Twitter: https://twitter.com/shadowtzu
'Twitch: http://www.twitch.tv/shadowtzu

Imports System.Windows.Media.MediaPlayer
Public Class TZSound
    Implements IDisposable

    ''' <summary>
    ''' structure des sons
    ''' </summary>
    ''' <remarks></remarks>
    Private Structure struct_Sound
        Public filename As String
        Public IsLoop As Boolean
    End Structure

    ''' <summary>
    ''' structure des players
    ''' </summary>
    ''' <remarks></remarks>
    Private Structure struct_Player
        Public wmp As Windows.Media.MediaPlayer
        Public IsOccuped As Boolean        
        Public SoundName As String
    End Structure

    ''' <summary>
    ''' liste des sons
    ''' </summary>
    ''' <remarks></remarks>
    Private mSound_List As Hashtable

    ''' <summary>
    ''' File d'attente des sons à jouer
    ''' </summary>
    ''' <remarks></remarks>
    Private Task_Play As Queue(Of String)

    ''' <summary>
    ''' liste des players
    ''' </summary>
    ''' <remarks>Count = MaxSound</remarks>
    Private mMedialPlayer_List() As struct_Player
    Private mMaxSound As Integer

    'variable privé
    Private mCurrent_Index As Integer
    Private mcurrent_Sound As struct_Sound
    Private mCurrent_SoundName As String   

    Private mOccupedCount As Integer
    Private mMaxSoundStep As Integer
    Private mInc As Integer

    ''' <summary>
    ''' Constructeur
    ''' </summary>
    ''' <param name="MaxSound">Nombre maximum de son à jouer en même temps</param>
    ''' <remarks></remarks>
    Public Sub New(MaxSound As Integer)
        mMaxSound = MaxSound
        mSound_List = New Hashtable
        ReDim mMedialPlayer_List(MaxSound - 1)
        For i As Integer = 0 To MaxSound - 1
            mMedialPlayer_List(i).wmp = New Windows.Media.MediaPlayer
        Next
        Task_Play = New Queue(Of String)
    End Sub

    ''' <summary>
    ''' Ajoute un son à la liste
    ''' </summary>
    ''' <param name="Filename">Emplacement du fichier son</param>
    ''' <param name="IsLoop">Jouer en boucle</param>
    ''' <param name="Name">Nom unique</param>
    ''' <remarks></remarks>
    Public Sub Add(Filename As String, IsLoop As Boolean, Name As String)
        Dim current_Sound As struct_Sound
        current_Sound.filename = IO.Path.GetFullPath(Filename)
        current_Sound.IsLoop = IsLoop

        mSound_List.Add(Name, current_Sound)
    End Sub

    ''' <summary>
    ''' joue un son à partir de son nom
    ''' </summary>
    ''' <param name="Name"></param>
    ''' <remarks></remarks>
    Public Sub Play(Name As String)
        Task_Play.Enqueue(Name)
    End Sub

    ''' <summary>
    ''' Joue un son à partir de son nom
    ''' </summary>
    ''' <param name="Name">Nom du son</param>
    ''' <param name="isLoop">Jouer en boucle</param>
    ''' <remarks></remarks>
    Public Sub Play(Name As String, isLoop As Boolean)
        Task_Play.Enqueue(Name)
        mcurrent_Sound = CType(mSound_List(Name), struct_Sound)
        mcurrent_Sound.IsLoop = isLoop
        mSound_List(Name) = mcurrent_Sound
    End Sub

    ''' <summary>
    ''' Processus de lecture des sons
    ''' </summary>
    ''' <remarks>à utiliser dans une boucle</remarks>
    Public Sub Process()
        mMaxSoundStep = Math.Min(mMaxSound - 1, Task_Play.Count + mOccupedCount)

        'parcourir la liste des players
        For Me.mInc = 0 To mMaxSoundStep

            With mMedialPlayer_List(mInc)

                'si le player actuel est occupé, vérifier s'il faut jouer le son actuel en boucle ou bien si le son à été joué entièrement
                If .IsOccuped = True Then
                    mcurrent_Sound = CType(mSound_List(.SoundName), struct_Sound)
                    If mcurrent_Sound.IsLoop Then
                        loop_sound(.wmp)
                    ElseIf IsEndPosition(.wmp) Then
                        .wmp.Close()
                        .IsOccuped = False
                        mOccupedCount -= 1
                    End If

                    'si le player est libre et qu'il reste des sons dans la file d'attente alors les jouer
                ElseIf .IsOccuped = False AndAlso Task_Play.Count > 0 Then
                    mCurrent_SoundName = Task_Play.Peek
                    mcurrent_Sound = CType(mSound_List(Task_Play.Dequeue), struct_Sound)

                    .IsOccuped = True
                    .wmp.Open(New Uri(mcurrent_Sound.filename))
                    .SoundName = mCurrent_SoundName
                    .wmp.Play()
                    mOccupedCount += 1
                End If

            End With

        Next
    End Sub

    ''' <summary>
    ''' Nombre de son dans la file d'attente
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private ReadOnly Property TaskCounts() As Integer
        Get
            If Not Task_Play Is Nothing Then
                Return Task_Play.Count
            Else
                Return 0
            End If
        End Get
    End Property

    ''' <summary>
    ''' joue un son en boucle en repositionnant la position de lecture au début
    ''' </summary>
    ''' <param name="wmp"></param>
    ''' <remarks></remarks>
    Private Sub loop_sound(ByRef wmp As Windows.Media.MediaPlayer)
        With wmp
            If .NaturalDuration.HasTimeSpan Then
                If .Position >= .NaturalDuration.TimeSpan Then
                    .Position = TimeSpan.Zero
                End If
            End If
        End With

    End Sub

    ''' <summary>
    ''' indique si le son à été joué dans son intégralité
    ''' </summary>
    ''' <param name="wmp"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Function IsEndPosition(ByRef wmp As Windows.Media.MediaPlayer) As Boolean
        With wmp
            If .NaturalDuration.HasTimeSpan Then
                If .Position >= .NaturalDuration.TimeSpan Then
                    Return True
                End If
            End If
        End With

        Return False
    End Function


#Region "IDisposable Support"
    Private disposedValue As Boolean ' Pour détecter les appels redondants

    ' IDisposable
    Protected Overridable Sub Dispose(disposing As Boolean)
        If Not Me.disposedValue Then
            If disposing Then
                ' TODO: supprimez l'état managé (objets managés).
            End If

            For i As Integer = 0 To mMedialPlayer_List.Length - 1
                mMedialPlayer_List(i).wmp.Close()
                mMedialPlayer_List(i).wmp = Nothing
            Next
            Erase mMedialPlayer_List
            Task_Play.Clear()
            Task_Play = Nothing

            mSound_List.Clear()
            mSound_List = Nothing
        End If
        Me.disposedValue = True
    End Sub

    ' TODO: substituez Finalize() uniquement si Dispose(ByVal disposing As Boolean) ci-dessus comporte du code permettant de libérer des ressources non managées.
    'Protected Overrides Sub Finalize()
    '    ' Ne modifiez pas ce code. Ajoutez du code de nettoyage dans Dispose(ByVal disposing As Boolean) ci-dessus.
    '    Dispose(False)
    '    MyBase.Finalize()
    'End Sub

    ' Ce code a été ajouté par Visual Basic pour permettre l'implémentation correcte du modèle pouvant être supprimé.
    Public Sub Dispose() Implements IDisposable.Dispose
        ' Ne modifiez pas ce code. Ajoutez du code de nettoyage dans Dispose(ByVal disposing As Boolean) ci-dessus.
        Dispose(True)
        GC.SuppressFinalize(Me)
    End Sub
#End Region

End Class
